//  Previous name:  MediaGlow2011_4_3.pov
#version 3.7;                   

#declare Radiosity = 1;

global_settings
{
    assumed_gamma 1.0
    #if (Radiosity)
    
        radiosity
        {
          pretrace_start 0.08           // start pretrace at this size
          pretrace_end   0.04           // end pretrace at this size
          count 35                      // higher -> higher quality (1..1600) [35]
          nearest_count 5               // higher -> higher quality (1..10) [5]
          error_bound 1.8               // higher -> smoother, less accurate [1.8]
          recursion_limit 3             // how much interreflections are calculated (1..5+) [3]
          low_error_factor 0.5          // reduce error_bound during last pretrace step
          gray_threshold 0.0            // increase for weakening colors (0..1) [0]
          minimum_reuse 0.015           // reuse of old radiosity samples [0.015]
          brightness 1                  // brightness of radiosity effects (0..1) [1]
    
          adc_bailout 0.01/2
          normal on                   // take surface normals into account [off]
          media on                    // take media into account [off]
          //save_file "file_name"       // save radiosity data
          //load_file "file_name"       // load saved radiosity data                   
          always_sample off           // turn sampling in final trace off [on]
          max_sample 1.0              // maximum brightness of samples
        }
        
        /*
        radiosity
        {
            pretrace_start 0.08
            pretrace_end 0.02
            count 35
            error_bound 0.5
            recursion_limit 1
            media on
        }*/
        /**/
    #end
}

#default 
{
    texture 
    {
        pigment 
        {
            rgb 0.06125
        }
        #if (Radiosity)
            finish 
            {
                ambient 0.0
                diffuse 0.6
                specular 0.03
            }
        #else
            finish 
            {
                ambient 0.1
                diffuse 0.6
                specular 0.3
            }
        #end
    }
}

#declare cameraLocation = <-5, 6, -18>;
#declare cameraLookAt = <0, 1.5, 0>;

camera
{
	location cameraLocation
	up y*image_height right x*image_width
	look_at cameraLookAt
	angle 45
}

// ----------------------------------------

#macro PrintVector(v0)
    #debug concat("<", vstr(3, v0, ", ", 0, -1), "> \n")
#end

//  IsWithinRange_Value Macro by [GDS|Entropy] Ian McDonald 
#macro IsWithinRange_Value(Value, Min, Max)
    #local result = false;
    
    #if (Value < Max & Value > Min)
        #local result = true;
    #end
    
    result
#end

//  PrintOperations Macro by [GDS|Entropy] Ian McDonald
#macro PrintOperations(Value) 
    #debug concat("------------------------------ \n")
    #debug concat("[ Value ", str(Value, 20, 20), "] \n")
    #debug concat("[ exp ", str(exp(Value), 20, 20), "] \n")
    #debug concat("[ log ", str(log(Value), 20, 20), "] \n")
    #debug concat("[ ln ", str(ln(Value), 20, 20), "] \n")
    #debug concat("[ pow(log(Value), Value) ", str(pow(log(Value), Value), 20, 20), "] \n")
    #debug concat("[ pow(Value, 2) ", str(pow(Value, 2),0,0), "] \n")
    #debug concat("[ sin ", str(sin(Value), 20, 20), "] \n")
    #debug concat("[ sinh ", str(sinh(Value), 20, 20), "] \n")
    #debug concat("[ cos ", str(cos(Value), 20, 20), "] \n")
    #debug concat("[ cosh ", str(cosh(Value), 20, 20), "] \n")
    #debug concat("[ tan ", str(tan(Value), 20, 20), "] \n")
    #debug concat("[ tanh ", str(tanh(Value), 20, 200), "] \n")                                            
    #debug concat("[ sqrt ", str(sqrt(Value), 20, 20), "] \n")
    #debug concat("------------------------------ \n")
#end

//  Calculate Percent Macro by [GDS|Entropy] Ian Mcdonald
#macro CalculatePercent(Value, Total)
    ((Value / Total) * 100)
#end

// ----------------------------------------
 
#macro BlackHoleWarp(Iterations, Falloff, Strength, Position, Radius)
    #local counter = 0; // set counter to 0
    #while (counter < Iterations) 
   
        warp 
        {
            black_hole Position, Radius
            falloff Falloff
            strength Strength
            turbulence 0.4
        }
   
        #local counter = counter + 1;
    #end
#end
		
#macro Glow(glowPosition, glowRadius, glowColor)
#local coreScaleFactor = 0.00765625;
#local density0 = true;
#local density1 = true;
#local density2 = true;

#local enableSurroundingFog = true;
#local enablePrimaryHalo = true;
#local enableCoreFuzz = true;
#local enableBrightCenter = true;

#if (enableSurroundingFog)
//  Surrounding "Fog"
light_source 
{
    glowPosition       
    color rgb glowColor
    fade_distance 4 
    fade_power 2
    looks_like 
    { 
        sphere 
        { 
            <0, 0, 0>, glowRadius * 50
            hollow 
            pigment 
            { 
                rgbt <0, 0, 0, 1> 
            }
            interior 
            {
                media 
                {
                    intervals 5         
                    samples 100, 500
                    //samples 10, 100          
                    emission glowColor
                    method 2
                    #if (density0) 
                        density
                        {
                            spherical
                            scale 8
                            density_map
                            {                         
                                [0.0 rgb glowColor * 0.0000]
                                [0.8 rgb glowColor * 0.00765625]
                            }                        
                        }
                    #end
                }
            }
            finish
            {
                reflection 0
                diffuse 0
                ambient 0
                phong 0
                specular 0
                brilliance 0                
            }             
        } 
    } 
}
#end

#if (enablePrimaryHalo)
//  Primary "Halo"
light_source 
{
    glowPosition       
    color rgb glowColor
    fade_distance 4 
    fade_power 3
    looks_like 
    { 
        sphere 
        { 
            <0, 0, 0>, glowRadius
            hollow 
            pigment 
            { 
                rgbt <0, 0, 0, 1> 
            }
            interior 
            {
                media 
                {
                    intervals 5         
                    //samples 250, 500          
                    samples 10, 100
                    emission glowColor
                    method 2
                    #if (density0) 
                        density
                        {
                            spherical
                            scale 3
                            density_map
                            {                         
                                [0.0 rgb glowColor * 0.000]
                                [0.8 rgb glowColor * 0.250]
                                [0.9 rgb glowColor * 0.750]
                                [1.0 rgb glowColor * 1.000]                        
                            }                        
                        }
                    #end
                    #if (density1)
                        density
                        { 
                            spherical
                            density_map
                            {
                                [0.00 color rgb glowColor * 0.000]
                                [ 0.90
                                    granite
                                    color_map 
                                    {
                                        [0 color rgb glowColor * 0.750]
                                        [1 color rgb glowColor * 0.250]
                                    }
                                    BlackHoleWarp(4, 1, 1, <0,0,0>, 1)
                                ]
                                [1.00 color rgb glowColor * 1.500]                        
                            }                                                                      
                        } 
                    #end 
                    #if (density2)
                        density
                        { 
                            spherical
                            density_map
                            {
                                [0.00 color rgb glowColor * 0.000]
                                [ 0.90
                                    granite
                                    color_map 
                                    {
                                        [0 color rgb glowColor * 2.750]
                                        [1 color rgb glowColor * 2.250]
                                    }
                                ]
                                [1.00 color rgb glowColor * 2.500]                        
                            }                                                                      
                        } 
                    #end                                       
                }
            }
            finish
            {
                reflection 0
                diffuse 0
                ambient 0
                phong 0
                specular 0
                brilliance 0                
            }             
        } 
    } 
}
#end

#if (enableCoreFuzz)
//  Core "Fuzz"
light_source 
{
    glowPosition       
    color rgb glowColor
    fade_distance 4 
    fade_power 4
    looks_like 
    { 
        sphere 
        { 
            <0, 0, 0>, (glowRadius * (coreScaleFactor * 2))
            hollow 
            pigment 
            { 
                rgbt <0, 0, 0, 1> 
            }
            interior 
            {
                media 
                {
                    intervals 5         
                    //samples 100, 500
                    samples 10, 100          
                    emission glowColor
                    method 2
                    #if (density0) 
                        density
                        {
                            spherical
                            density_map
                            {                         
                                [0.0 rgb glowColor * 0.0000]
                                [0.8 rgb glowColor * 10]
                            }                        
                        }
                    #end
                }
            }
            finish
            {
                reflection 0
                diffuse 0
                ambient 0
                phong 0
                specular 0
                brilliance 0                
            }             
        } 
    } 
}
#end

#if (enableBrightCenter)
//  Bright Center
sphere 
{ 
    glowPosition, glowRadius * coreScaleFactor
    hollow 
    pigment 
    { 
        color rgb glowColor * 1.5 
    }
    finish
    {
        emission glowColor
        ambient 0
        diffuse 1 
        reflection 0
        phong 0
        specular 0
        brilliance 0                                
    }
}
#end

#end 
// ---------------------------------------- 
  
   
//Types:
#local Float = 0;
#local Vector = < 0, 0, 0>;
#local Color = color rgb < 0, 0, 0>;

#macro SRand(RS) (rand(RS)*2 - 1) #end
#macro RRand(RS, Min, Max) (rand(RS)*(Max-Min) + Min) #end
#macro Clamp(V, Min, Max) (min(Max, max(Min, V))) #end
#macro Range(V, Rmn, Rmx) (V*(Rmx-Rmn) + Rmn) #end
#macro RClamp(V, Rmn, Rmx, Min, Max) (Clamp(Range(V, Rmn, Rmx), Min, Max)) #end

#local RsA = seed(574647);

#local J = Float;
#local J = 0;
#while(J<52)
    Glow(< SRand(RsA) * 8, rand(RsA) * 4, SRand(RsA) * 8>, rand(RsA) * 1.5, < RRand(RsA, 0.1, 1), RRand(RsA, 0.1, 1), RRand(RsA, 0.1, 1)> * 1.0)
	#local J = J + 1;
#end

box
{
	<-15, 0,-10>,
	< 15, 0, 10>
	pigment
	{
	    checker
	    color rgb 0.05,
	    color rgb 1
	}
	finish
	{
		reflection
		{
		    0.5, 0.75 metallic
		}
		diffuse 0.25
		ambient 0
	}
}

sphere
{
	< 0, 2, 0>, 2
	pigment
	{
	    rgb 1
	}
	finish
	{
		reflection
		{
		    0.5, 0.85
		    metallic
		}
		diffuse 0.15
		ambient 0
	}
}
